@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -26,30 +26,33 @@ require_once dirname(__FILE__).'/include/forms.inc.php';
 
 $page['title'] = _('Configuration of host prototypes');
 $page['file'] = 'host_prototypes.php';
-$page['scripts'] = ['effects.js', 'class.cviewswitcher.js', 'multiselect.js', 'textareaflexible.js'];
+$page['scripts'] = ['effects.js', 'class.cviewswitcher.js', 'multiselect.js', 'textareaflexible.js',
+	'class.cverticalaccordion.js', 'inputsecret.js', 'macrovalue.js'
+];
 
 require_once dirname(__FILE__).'/include/page_header.php';
 
 // VAR	TYPE	OPTIONAL	FLAGS	VALIDATION	EXCEPTION
 $fields = [
-	'hostid' =>					[T_ZBX_INT, O_NO,	P_SYS,	DB_ID,		'(isset({form}) && ({form} == "update"))'],
+	'hostid' =>					[T_ZBX_INT, O_NO,	P_SYS,	DB_ID, '(isset({form}) && ({form} == "update")) || (isset({action}) && {action} == "hostprototype.updatediscover")'],
 	'parent_discoveryid' =>		[T_ZBX_INT, O_MAND, P_SYS,	DB_ID, null],
-	'host' =>		        	[T_ZBX_STR, O_OPT, null,		NOT_EMPTY,	'isset({add}) || isset({update})', _('Host name')],
-	'name' =>	            	[T_ZBX_STR, O_OPT, null,		null,		'isset({add}) || isset({update})'],
-	'status' =>		        	[T_ZBX_INT, O_OPT, null,		IN([HOST_STATUS_NOT_MONITORED, HOST_STATUS_MONITORED]), null],
+	'host' =>					[T_ZBX_STR, O_OPT, null,		NOT_EMPTY,	'isset({add}) || isset({update})', _('Host name')],
+	'name' =>					[T_ZBX_STR, O_OPT, null,		null,		'isset({add}) || isset({update})'],
+	'status' =>					[T_ZBX_INT, O_OPT, null,		IN([HOST_STATUS_NOT_MONITORED, HOST_STATUS_MONITORED]), null],
+	'discover' =>				[T_ZBX_INT, O_OPT, null, IN([ZBX_PROTOTYPE_DISCOVER, ZBX_PROTOTYPE_NO_DISCOVER]), null],
 	'inventory_mode' =>			[T_ZBX_INT, O_OPT, null, IN([HOST_INVENTORY_DISABLED, HOST_INVENTORY_MANUAL, HOST_INVENTORY_AUTOMATIC]), null],
-	'templates' =>		    	[T_ZBX_STR, O_OPT, null, NOT_EMPTY,	null],
-	'add_template' =>			[T_ZBX_STR, O_OPT, null,		null,	null],
-	'add_templates' =>		    [T_ZBX_STR, O_OPT, null, NOT_EMPTY,	null],
+	'templates' =>				[T_ZBX_STR, O_OPT, null, NOT_EMPTY,	null],
+	'add_templates' =>			[T_ZBX_STR, O_OPT, null, NOT_EMPTY,	null],
 	'group_links' =>			[T_ZBX_STR, O_OPT, null, NOT_EMPTY,	null],
 	'group_prototypes' =>		[T_ZBX_STR, O_OPT, null, NOT_EMPTY,	null],
 	'unlink' =>					[T_ZBX_STR, O_OPT, P_SYS|P_ACT,	null,		null],
 	'group_hostid' =>			[T_ZBX_INT, O_OPT, null,	DB_ID,		null],
 	'show_inherited_macros' =>	[T_ZBX_INT, O_OPT, null, IN([0,1]), null],
+	'macros' =>					[T_ZBX_STR, O_OPT, P_SYS,			null,		null],
 	// actions
 	'action' =>					[T_ZBX_STR, O_OPT, P_SYS|P_ACT,
 									IN('"hostprototype.massdelete","hostprototype.massdisable",'.
-										'"hostprototype.massenable"'
+										'"hostprototype.massenable","hostprototype.updatediscover"'
 									),
 									null
 								],
@@ -61,7 +64,7 @@ $fields = [
 	'form' =>					[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
 	'form_refresh' =>			[T_ZBX_INT, O_OPT, null,	null,		null],
 	// sort and sortorder
-	'sort' =>					[T_ZBX_STR, O_OPT, P_SYS, IN('"name","status"'),						null],
+	'sort' =>					[T_ZBX_STR, O_OPT, P_SYS, IN('"name","status","discover"'),						null],
 	'sortorder' =>				[T_ZBX_STR, O_OPT, P_SYS, IN('"'.ZBX_SORT_DOWN.'","'.ZBX_SORT_UP.'"'),	null]
 ];
 check_fields($fields);
@@ -87,6 +90,7 @@ if (getRequest('parent_discoveryid')) {
 			'selectGroupPrototypes' => API_OUTPUT_EXTEND,
 			'selectTemplates' => ['templateid', 'name'],
 			'selectParentHost' => ['hostid'],
+			'selectMacros' => ['hostmacroid', 'macro', 'value', 'type', 'description'],
 			'editable' => true
 		]);
 		$hostPrototype = reset($hostPrototype);
@@ -99,17 +103,20 @@ else {
 	access_deny();
 }
 
+// Remove inherited macros data (actions: 'add', 'update' and 'form').
+$macros = cleanInheritedMacros(getRequest('macros', []));
+
+// Remove empty new macro lines.
+$macros = array_filter($macros, function($macro) {
+	$keys = array_flip(['hostmacroid', 'macro', 'value', 'description']);
+
+	return (bool) array_filter(array_intersect_key($macro, $keys));
+});
+
 /*
  * Actions
  */
-// add templates to the list
-if (getRequest('add_template')) {
-	foreach (getRequest('add_templates', []) as $templateId) {
-		$_REQUEST['templates'][$templateId] = $templateId;
-	}
-}
-// unlink templates
-elseif (getRequest('unlink')) {
+if (getRequest('unlink')) {
 	foreach (getRequest('unlink') as $templateId => $value) {
 		unset($_REQUEST['templates'][$templateId]);
 	}
@@ -134,6 +141,24 @@ elseif (isset($_REQUEST['clone']) && isset($_REQUEST['hostid'])) {
 		}
 		unset($groupPrototype);
 	}
+
+	if ($macros && in_array(ZBX_MACRO_TYPE_SECRET, array_column($macros, 'type'))) {
+		// Reset macro type and value.
+		$macros = array_map(function($value) {
+			return ($value['type'] == ZBX_MACRO_TYPE_SECRET)
+				? ['value' => '', 'type' => ZBX_MACRO_TYPE_TEXT] + $value
+				: $value;
+		}, $macros);
+
+		$msg = [
+			'type' => 'error',
+			'message' => _('The cloned host prototype contains user defined macros with type "Secret text". The value and type of these macros were reset.'),
+			'src' => ''
+		];
+
+		echo makeMessageBox(false, [$msg], null, true, false)->addClass(ZBX_STYLE_MSG_WARNING);
+	}
+
 	$_REQUEST['form'] = 'clone';
 }
 elseif (hasRequest('add') || hasRequest('update')) {
@@ -143,9 +168,11 @@ elseif (hasRequest('add') || hasRequest('update')) {
 		'host' => getRequest('host', ''),
 		'name' => (getRequest('name', '') === '') ? getRequest('host', '') : getRequest('name', ''),
 		'status' => getRequest('status', HOST_STATUS_NOT_MONITORED),
+		'discover' => getRequest('discover', DB::getDefault('hosts', 'discover')),
 		'groupLinks' => [],
 		'groupPrototypes' => [],
-		'templates' => getRequest('templates', [])
+		'macros' => $macros,
+		'templates' => array_merge(getRequest('templates', []), getRequest('add_templates', []))
 	];
 
 	if (hasRequest('inventory_mode')) {
@@ -208,6 +235,17 @@ elseif (hasRequest('add') || hasRequest('update')) {
 			];
 		}
 
+		/*
+		 * Sanitize macros array. When we clone, we have old hostmacroid.
+		 * We need delete them before we push array to API.
+		*/
+		foreach ($newHostPrototype['macros'] as &$macro) {
+			if (array_key_exists('hostmacroid', $macro)) {
+				unset($macro['hostmacroid']);
+			}
+		}
+		unset($macro);
+
 		$result = API::HostPrototype()->create($newHostPrototype);
 
 		show_messages($result, _('Host prototype added'), _('Cannot add host prototype'));
@@ -220,6 +258,14 @@ elseif (hasRequest('add') || hasRequest('update')) {
 		unset($_REQUEST['itemid'], $_REQUEST['form']);
 	}
 }
+elseif (getRequest('hostid', '') && getRequest('action', '') === 'hostprototype.updatediscover') {
+	$result = API::HostPrototype()->update([
+		'hostid' => getRequest('hostid'),
+		'discover' => getRequest('discover', DB::getDefault('hosts', 'discover'))
+	]);
+
+	show_messages($result, _('Host prototype updated'), _('Cannot update host prototype'));
+}
 // GO
 elseif (hasRequest('action') && str_in_array(getRequest('action'), ['hostprototype.massenable', 'hostprototype.massdisable']) && hasRequest('group_hostid')) {
 	$status = (getRequest('action') == 'hostprototype.massenable') ? HOST_STATUS_MONITORED : HOST_STATUS_NOT_MONITORED;
@@ -280,32 +326,54 @@ if (hasRequest('form')) {
 			'host' => getRequest('host'),
 			'name' => getRequest('name'),
 			'status' => getRequest('status', HOST_STATUS_NOT_MONITORED),
+			'discover' => getRequest('discover', DB::getDefault('hosts', 'discover')),
 			'templates' => [],
+			'add_templates' => [],
 			'inventory_mode' => getRequest('inventory_mode', $config['default_inventory_mode']),
-			'groupPrototypes' => getRequest('group_prototypes', [])
+			'groupPrototypes' => getRequest('group_prototypes', []),
+			'macros' => $macros
 		],
-		'groups' => [],
 		'show_inherited_macros' => getRequest('show_inherited_macros', 0),
+		'readonly' => (getRequest('hostid') && $hostPrototype['templateid']),
+		'groups' => [],
+		// Parent discovery rules.
 		'templates' => []
 	];
 
-	// add already linked and new templates
-	$data['host_prototype']['templates'] = API::Template()->get([
-		'output' => ['templateid', 'name'],
-		'templateids' => getRequest('templates', [])
-	]);
+	// Add already linked and new templates.
+	$templates = [];
+	$request_templates = getRequest('templates', []);
+	$request_add_templates = getRequest('add_templates', []);
+
+	if ($request_templates || $request_add_templates) {
+		$templates = API::Template()->get([
+			'output' => ['templateid', 'name'],
+			'templateids' => array_merge($request_templates, $request_add_templates),
+			'preservekeys' => true
+		]);
+
+		$data['host_prototype']['templates'] = array_intersect_key($templates, array_flip($request_templates));
+		CArrayHelper::sort($data['host_prototype']['templates'], ['name']);
+
+		$data['host_prototype']['add_templates'] = array_intersect_key($templates, array_flip($request_add_templates));
+
+		foreach ($data['host_prototype']['add_templates'] as &$template) {
+			$template = CArrayHelper::renameKeys($template, ['templateid' => 'id']);
+		}
+		unset($template);
+	}
 
 	// add parent host
 	$parentHost = API::Host()->get([
 		'output' => API_OUTPUT_EXTEND,
 		'selectGroups' => ['groupid', 'name'],
 		'selectInterfaces' => API_OUTPUT_EXTEND,
-		'selectMacros' => ['macro', 'value', 'description'],
 		'hostids' => $discoveryRule['hostid'],
 		'templated_hosts' => true
 	]);
 	$parentHost = reset($parentHost);
 	$data['parent_host'] = $parentHost;
+	$data['parent_hostid'] = $parentHost['hostid'];
 
 	if (getRequest('group_links')) {
 		$data['groups'] = API::HostGroup()->get([
@@ -373,13 +441,33 @@ if (hasRequest('form')) {
 		]);
 	}
 
-	// order linked templates
-	CArrayHelper::sort($data['host_prototype']['templates'], ['name']);
+	$macros = $data['host_prototype']['macros'];
 
-	// render view
-	$itemView = new CView('configuration.host.prototype.edit', $data);
-	$itemView->render();
-	$itemView->show();
+	if ($data['show_inherited_macros']) {
+		$macros = mergeInheritedMacros($macros, getInheritedMacros(array_keys($templates), $data['parent_hostid']));
+	}
+
+	// Sort only after inherited macros are added. Otherwise the list will look chaotic.
+	$data['macros'] = array_values(order_macros($macros, 'macro'));
+
+	if (!$data['macros'] && !$data['readonly']) {
+		$macro = ['macro' => '', 'value' => '', 'description' => '', 'type' => ZBX_MACRO_TYPE_TEXT];
+
+		if ($data['show_inherited_macros']) {
+			$macro['inherited_type'] = ZBX_PROPERTY_OWN;
+		}
+
+		$data['macros'][] = $macro;
+	}
+
+	// This data is used in common.template.edit.js.php.
+	$data['macros_tab'] = [
+		'linked_templates' => array_map('strval', $templateids),
+		'add_templates' => array_map('strval', array_keys($data['host_prototype']['add_templates']))
+	];
+
+	// Render view.
+	echo (new CView('configuration.host.prototype.edit', $data))->getOutput();
 }
 else {
 	$sortField = getRequest('sort', CProfile::get('web.'.$page['file'].'.sort', 'name'));
@@ -408,10 +496,22 @@ else {
 
 	order_result($data['hostPrototypes'], $sortField, $sortOrder);
 
-	$url = (new CUrl('host_prototypes.php'))
-		->setArgument('parent_discoveryid', $data['parent_discoveryid']);
+	// pager
+	if (hasRequest('page')) {
+		$page_num = getRequest('page');
+	}
+	elseif (isRequestMethod('get') && !hasRequest('cancel')) {
+		$page_num = 1;
+	}
+	else {
+		$page_num = CPagerHelper::loadPage($page['file']);
+	}
 
-	$data['paging'] = getPagingLine($data['hostPrototypes'], $sortOrder, $url);
+	CPagerHelper::savePage($page['file'], $page_num);
+
+	$data['paging'] = CPagerHelper::paginate($page_num, $data['hostPrototypes'], $sortOrder,
+		(new CUrl('host_prototypes.php'))->setArgument('parent_discoveryid', $data['parent_discoveryid'])
+	);
 
 	$data['parent_templates'] = getHostPrototypeParentTemplates($data['hostPrototypes']);
 
@@ -448,9 +548,7 @@ else {
 	}
 
 	// render view
-	$itemView = new CView('configuration.host.prototype.list', $data);
-	$itemView->render();
-	$itemView->show();
+	echo (new CView('configuration.host.prototype.list', $data))->getOutput();
 }
 
 require_once dirname(__FILE__).'/include/page_footer.php';
