@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,6 +19,10 @@
 **/
 
 
+/**
+ * @var CView $this
+ */
+
 $discoveryRule = $data['discovery_rule'];
 $hostPrototype = $data['host_prototype'];
 $parentHost = $data['parent_host'];
@@ -32,6 +36,7 @@ $widget = (new CWidget())
 	->addItem(get_header_host_table('hosts', $discoveryRule['hostid'], $discoveryRule['itemid']));
 
 $divTabs = new CTabView();
+
 if (!hasRequest('form_refresh')) {
 	$divTabs->setSelected(0);
 }
@@ -67,92 +72,41 @@ $visiblenameTB = (new CTextBox('name', $name, (bool) $hostPrototype['templateid'
 	->setAttribute('maxlength', 128);
 $hostList->addRow(_('Visible name'), $visiblenameTB);
 
-// display inherited parameters only for hosts prototypes on hosts
+// Display inherited parameters only for hosts prototypes on hosts.
 if ($parentHost['status'] != HOST_STATUS_TEMPLATE) {
 	$existingInterfaceTypes = [];
 	foreach ($parentHost['interfaces'] as $interface) {
 		$existingInterfaceTypes[$interface['type']] = true;
 	}
 
-	zbx_add_post_js('hostInterfacesManager.add('.CJs::encodeJson(array_values($parentHost['interfaces'])).');');
-	zbx_add_post_js('hostInterfacesManager.disable();');
+	zbx_add_post_js('window.hostInterfaceManager = new HostInterfaceManager('.json_encode(array_values($parentHost['interfaces'])).');');
+	zbx_add_post_js('hostInterfaceManager.render();');
+	zbx_add_post_js('HostInterfaceManager.disableEdit();');
 
-	// Zabbix agent interfaces
-	$ifTab = (new CTable())
+	$interface_header = renderInterfaceHeaders();
+
+	$agent_interfaces = (new CDiv())
 		->setId('agentInterfaces')
-		->setHeader([
-			'',
-			_('IP address'),
-			_('DNS name'),
-			_('Connect to'),
-			_('Port'),
-			(new CColHeader(_('Default')))->setColSpan(2)
-		]);
+		->addClass(ZBX_STYLE_HOST_INTERFACE_CONTAINER)
+		->setAttribute('data-type', 'agent');
 
-	$row = (new CRow())->setId('agentInterfacesFooter');
-	if (!array_key_exists(INTERFACE_TYPE_AGENT, $existingInterfaceTypes)) {
-		$row->addItem(new CCol());
-		$row->addItem((new CCol(_('No agent interfaces found.')))->setColSpan(6));
-	}
-	$ifTab->addRow($row);
+	$snmp_interfaces = (new CDiv())
+		->setId('SNMPInterfaces')
+		->addClass(ZBX_STYLE_HOST_INTERFACE_CONTAINER.' '.ZBX_STYLE_LIST_VERTICAL_ACCORDION)
+		->setAttribute('data-type', 'snmp');
 
-	$hostList->addRow(_('Agent interfaces'),
-		(new CDiv($ifTab))
-			->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
-			->setAttribute('data-type', 'agent')
-			->setWidth(ZBX_HOST_INTERFACE_WIDTH)
-	);
+	$jmx_interfaces = (new CDiv())
+		->setId('JMXInterfaces')
+		->addClass(ZBX_STYLE_HOST_INTERFACE_CONTAINER)
+		->setAttribute('data-type', 'jmx');
 
-	// SNMP interfaces
-	$ifTab = (new CTable())->setId('SNMPInterfaces');
+	$ipmi_interfaces = (new CDiv())
+		->setId('IPMIInterfaces')
+		->addClass(ZBX_STYLE_HOST_INTERFACE_CONTAINER)
+		->setAttribute('data-type', 'ipmi');
 
-	$row = (new CRow())->setId('SNMPInterfacesFooter');
-	if (!array_key_exists(INTERFACE_TYPE_SNMP, $existingInterfaceTypes)) {
-		$row->addItem(new CCol());
-		$row->addItem((new CCol(_('No SNMP interfaces found.')))->setColSpan(6));
-	}
-	$ifTab->addRow($row);
-
-	$hostList->addRow(_('SNMP interfaces'),
-		(new CDiv($ifTab))
-			->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
-			->setAttribute('data-type', 'snmp')
-			->setWidth(ZBX_HOST_INTERFACE_WIDTH)
-	);
-
-	// JMX interfaces
-	$ifTab = (new CTable())->setId('JMXInterfaces');
-
-	$row = (new CRow())->setId('JMXInterfacesFooter');
-	if (!array_key_exists(INTERFACE_TYPE_JMX, $existingInterfaceTypes)) {
-		$row->addItem(new CCol());
-		$row->addItem((new CCol(_('No JMX interfaces found.')))->setColSpan(6));
-	}
-	$ifTab->addRow($row);
-
-	$hostList->addRow(_('JMX interfaces'),
-		(new CDiv($ifTab))
-			->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
-			->setAttribute('data-type', 'jmx')
-			->setWidth(ZBX_HOST_INTERFACE_WIDTH)
-	);
-
-	// IPMI interfaces
-	$ifTab = (new CTable())->setId('IPMIInterfaces');
-
-	$row = (new CRow())->setId('IPMIInterfacesFooter');
-	if (!array_key_exists(INTERFACE_TYPE_IPMI, $existingInterfaceTypes)) {
-		$row->addItem(new CCol());
-		$row->addItem((new CCol(_('No IPMI interfaces found.')))->setColSpan(6));
-	}
-	$ifTab->addRow($row);
-
-	$hostList->addRow(
-		_('IPMI interfaces'),
-		(new CDiv($ifTab))
-			->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
-			->setAttribute('data-type', 'ipmi')
-			->setWidth(ZBX_HOST_INTERFACE_WIDTH)
+	$hostList->addRow(new CLabel(_('Interfaces')),
+		[new CDiv([$interface_header, $agent_interfaces, $snmp_interfaces, $jmx_interfaces, $ipmi_interfaces])]
 	);
 
 	// proxy
@@ -167,6 +121,12 @@ $hostList->addRow(_('Create enabled'),
 	(new CCheckBox('status', HOST_STATUS_MONITORED))
 		->setChecked(HOST_STATUS_MONITORED == $hostPrototype['status'])
 );
+$hostList->addRow(_('Discover'),
+	(new CRadioButtonList('discover', (int) $hostPrototype['discover']))
+		->addValue(_('Yes'), ZBX_PROTOTYPE_DISCOVER)
+		->addValue(_('No'), ZBX_PROTOTYPE_NO_DISCOVER)
+		->setModern(true)
+);
 
 $divTabs->addTab('hostTab', _('Host'), $hostList);
 
@@ -230,7 +190,7 @@ if ($hostPrototype['templateid']) {
 		->setHeader([_('Name')]);
 
 	foreach ($hostPrototype['templates'] as $template) {
-		$tmplList->addVar('templates['.$template['templateid'].']', $template['templateid']);
+		$tmplList->addItem((new CVar('templates['.$template['templateid'].']', $template['templateid']))->removeId());
 
 		if (array_key_exists($template['templateid'], $hostPrototype['writable_templates'])) {
 			$template_link = (new CLink($template['name'],
@@ -258,7 +218,7 @@ else {
 		->setHeader([_('Name'), _('Action')]);
 
 	foreach ($hostPrototype['templates'] as $template) {
-		$tmplList->addVar('templates['.$template['templateid'].']', $template['templateid']);
+		$tmplList->addItem((new CVar('templates['.$template['templateid'].']', $template['templateid']))->removeId());
 
 		if (array_key_exists($template['templateid'], $hostPrototype['writable_templates'])) {
 			$template_link = (new CLink($template['name'],
@@ -283,26 +243,35 @@ else {
 		$disableids[] = $template['templateid'];
 	}
 
-	$linkedTemplateTable->addRow([
-		(new CSimpleButton(_('Add')))
-			->onClick('return PopUp("popup.generic",'.CJs::encodeJson([
-				'dstfrm' => $frmHost->getName(),
-				'dstfld1' => $frmHost->getName(),
+	$add_templates_ms = (new CMultiSelect([
+		'name' => 'add_templates[]',
+		'object_name' => 'templates',
+		'data' => $hostPrototype['add_templates'],
+		'popup' => [
+			'parameters' => [
 				'srctbl' => 'templates',
 				'srcfld1' => 'hostid',
-				'templated_hosts' => '1',
-				'popup_type' => 'templates',
-				'multiselect' => 1,
+				'srcfld2' => 'host',
+				'dstfrm' => $frmHost->getName(),
+				'dstfld1' => 'add_templates_',
 				'disableids' => $disableids
-			]).', null, this);')
-			->addClass(ZBX_STYLE_BTN_LINK)
-	]);
+			]
+		]
+	]))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH);
 
-	$tmplList->addRow(_('Linked templates'),
-		(new CDiv($linkedTemplateTable))
-			->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
-			->setAttribute('style', 'min-width: '.ZBX_TEXTAREA_BIG_WIDTH.'px;')
-	);
+	$tmplList
+		->addRow(_('Linked templates'),
+			(new CDiv($linkedTemplateTable))
+				->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
+				->setAttribute('style', 'min-width: '.ZBX_TEXTAREA_BIG_WIDTH.'px;')
+		)
+		->addRow((new CLabel(_('Link new templates'), 'add_templates__ms')),
+			(new CDiv(
+				(new CTable())->addRow([$add_templates_ms])
+			))
+				->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
+				->setAttribute('style', 'min-width: '.ZBX_TEXTAREA_BIG_WIDTH.'px;')
+		);
 }
 
 $divTabs->addTab('templateTab', _('Templates'), $tmplList);
@@ -328,25 +297,24 @@ if ($parentHost['status'] != HOST_STATUS_TEMPLATE) {
 	);
 
 	$divTabs->addTab('ipmiTab', _('IPMI'), $ipmiList);
-
-	// macros
-	$macros = $parentHost['macros'];
-	if ($data['show_inherited_macros']) {
-		$macros = mergeInheritedMacros($macros,
-			getInheritedMacros(zbx_objectValues($hostPrototype['templates'], 'templateid'))
-		);
-	}
-	$macros = array_values(order_macros($macros, 'macro'));
-
-	$macrosView = new CView('hostmacros', [
-		'macros' => $macros,
-		'show_inherited_macros' => $data['show_inherited_macros'],
-		'is_template' => false,
-		'readonly' => true
-	]);
-	$divTabs->addTab('macroTab', _('Macros'), $macrosView->render());
 }
 
+// macros
+$tmpl = $data['show_inherited_macros'] ? 'hostmacros.inherited.list.html' : 'hostmacros.list.html';
+$divTabs->addTab('macroTab', _('Macros'),
+	(new CFormList('macrosFormList'))
+		->addRow(null, (new CRadioButtonList('show_inherited_macros', (int) $data['show_inherited_macros']))
+			->addValue(_('Host prototype macros'), 0)
+			->addValue(_('Inherited and host prototype macros'), 1)
+			->setModern(true)
+		)
+		->addRow(null, new CPartial($tmpl, [
+			'macros' => $data['macros'],
+			'parent_hostid' => $data['parent_host']['hostid'],
+			'readonly' => $data['readonly']
+		]), 'macros_container')
+);
+
 $inventoryFormList = (new CFormList('inventorylist'))
 	->addRow(null,
 		(new CRadioButtonList('inventory_mode', (int) $hostPrototype['inventory_mode']))
@@ -437,4 +405,4 @@ else {
 $frmHost->addItem($divTabs);
 $widget->addItem($frmHost);
 
-return $widget;
+$widget->show();
